library(jsonlite)
library(httr)
library(htmltidy)
library(XML)
library(xml2)
library(jsonview)
library(sf)

source('C:/Users/sea084/OneDrive - CSIRO/RossRCode/Git/TernLandscapes/APIs/SoilDataFederatoR/Demos/utils.R')



#########################################   Some web sites   ###################################################


###   SDF info page
browseURL('https://esoil.io/TERNLandscapes/Public/Pages/SoilDataFederator/SoilDataFederator.html')

###   SDF API help page
browseURL('https://esoil.io/TERNLandscapes/Public/Pages/SoilDataFederator/SoilDataFederatorHelp.html')

###   SDF registration page
browseURL('https://shiny.esoil.io/SoilDataFederator/Pages/Register/')

###   SDF registration page
browseURL('https://shiny.esoil.io/SoilDataFederator/Pages/Register/')

###   SDF Web App
browseURL('https://shiny.esoil.io/SoilDataFederator/Pages/App/')

###   SDF Swagger Docs
browseURL('https://esoil.io/TERNLandscapes/SoilDataFederatoR/__swagger__/')

###############################################################################################################






####################################   SDF API  ###############################################################

######  Querying for Soil Observed Properties   #######

usr <- 'TrustedDemo'
key <- 'jvdn64df'


# Get the raw json from the Datasets endpoint
rawJson <- content(GET('https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/DataSets'), "text" , encoding = 'UTF-8')
rawJson

# View the raw json data
json_view(rawJson)
json_tree_view(rawJson)

# Get the raw json from the Datasets endpoint and create a dataframe - This is probably the best way to use the SoilDataFederator endpoint in R
df <- fromJSON('https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/DataSets')
View(df)

# Get the Datasets data as XML
rawXML <- content(GET('https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/DataSets?format=xml'), "text")
rawXML
xml_view(rawXML)

# Get the Datasets data as XML and create a dataframe
doc<-xmlParse(rawXML)
xmldf <- xmlToDataFrame(nodes = getNodeSet(doc, "//DataSetsRecord"))
View(xmldf)

# Get the Datasets data as a CSV file
csvf <- tempfile()
csvf <-'c:/temp/aTestFile.csv'
download.file('https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/DataSets?format=csv', csvf, mode="wb", quiet = T)
df <- read.csv(csvf)
View(df)
unlink(csvf)



# get the soil properties from the Properties endpoint
url <- 'https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Properties'
df <- fromJSON(url)
View(df)



# get the soil property groups from the PropertyGroups endpoint
url <- 'https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/PropertyGroups'
df <- fromJSON(url)
View(df)




# get the soil property data from the SoilData endpoint for a single dataset for EC 1:5 Water
url <- paste0('https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/SoilData?observedProperty=3A1&DataSet=TasGovernment', '&usr=', usr, '&key=', key)
df <- fromJSON(url)
write.csv(df, 'c:/temp/aDataFile.csv')
shell.exec('c:/temp/aDataFile.csv')

####  Talk about the output data format and the usr/key


######  ******  DON'T RUN NEXT ONE IN PRESENTATION *****  #############

# get the soil property data from the SoilData endpoint for all DataSets for EC 1:5 Water
url <- paste0('https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/SoilData?observedProperty=3A1', '&usr=', usr, '&key=', key)
dfall <- fromJSON(url)
write.csv(dfall, 'c:/temp/allDataFile.csv')
shell.exec('c:/temp/allDataFile.csv')



#  Draw a map of results
dfall <- read.csv('c:/temp/SDF_DIST_TYPE.csv')
locs <- unique(dfall[c("Longitude", "Latitude", "Dataset", "Location_ID")])
nrow(locs)
pts <- st_as_sf(locs, coords = c("Longitude", "Latitude"), crs = 4326, na.fail = F)
drawLeafletMap(ptData=pts, field='Dataset', ptSize=2)




################################################    Get all soil property data for a single site   #####################################################

Longitude=140
Latitude=-30
bBox = '140;145;-35;30'

######  Get sites via a range of filter queries eg propertytype, bounding box, closest, within a radius

###  Return site information for a dataset
sdf <- fromJSON("http://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Locations?DataSets=NatSoil&usr=TrustedDemo&key=jvdn64df")
nrow(sdf)
head(sdf)


###  Return ALL site information - can take a few seconds. It is returning  298046 records
sdf <- fromJSON("http://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Locations?usr=TrustedDemo&key=jvdn64df")
nrow(sdf)
head(sdf)

###  Return only sites with lab data information for a dataset
sdf <- fromJSON("http://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Locations?DataSets=NatSoil&propertytype=LaboratoryMeasurement&usr=TrustedDemo&key=jvdn64df")
nrow(sdf)
head(sdf)

###  Return only sites within a bounding box minx=140 maxx=145 miny=-35 maxy=30
sdf <- fromJSON("http://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Locations?DataSets=NatSoil&propertytype=LaboratoryMeasurement&bbox=140;145;-35;-30&usr=TrustedDemo&key=jvdn64df")
nrow(sdf)
head(sdf)
drawLeafletMapFromDF(dfData=sdf, field='DataSet', ptSize=2)

###  Return sites within 100000 metres of the location provided
# sdf <- fromJSON("http://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Locations?radius=1000000&longitude=148&latitude=-25&usr=TrustedDemo&key=jvdn64df")
# nrow(sdf)
# head(sdf)
# drawLeafletMapFromDF(dfData=sdf, field='DataSet', ptSize=2)

###  Return the 20 closest sites
sdf <- fromJSON("https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Locations?longitude=130&latitude=-30&propertytype=LaboratoryMeasurement&closest=100&usr=TrustedDemo&key=jvdn64df")
nrow(sdf)
head(sdf)
drawLeafletMapFromDF(dfData=sdf, field='DataSet', ptSize=2)


#####  Now lets get the actual site data

####### get the data for a single site DataSet=NatSoil and siteid=199_CAN_C9_1

sdf <- fromJSON("https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Data?DataSet=NatSoil&siteid=199_CAN_C9_1&propertytype=LaboratoryMeasurement&usr=TrustedDemo&key=jvdn64df")
nrow(sdf)
View(sdf)


### There are two different formats for the data output table - 'narrow' or 'wide'
fromJSON("https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Data?DataSet=NatSoil&siteid=199_CAN_C9_1&propertytype=LaboratoryMeasurement&tabletype=wide&usr=TrustedDemo&key=jvdn64df")
fromJSON("https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/Site_Data?DataSet=NatSoil&siteid=199_CAN_C9_1&propertytype=LaboratoryMeasurement&tabletype=narrow&usr=TrustedDemo&key=jvdn64df")




##########   OK Back to the web App

###   SDF Web App
browseURL('https://shiny.esoil.io/SoilDataFederator/Pages/App/')









##################################################################################
#                                                                                #
#         Prefered approach to doing large data extractions from the SDF         #
#                                                                                #
##################################################################################


library(httr)
library(jsonlite)

# use your own usr & key
usr <- ''
key <- ''
usr <- 'TrustedDemo'
key <- 'jvdn64df'


# Get the available datasets
datasetsDF <- fromJSON(paste0('https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/DataSets?usr=', usr, '&key=', key))
write.csv(datasetsDF, 'c:/temp/SDFdatasets.csv', row.names = F)

datasets <- datasetsDF$DataSet

p <- 'DIST_TYPE'

# Make an empty list to put individual results in for this property
res <- vector("list", length = length(datasets))
# Iterate Datasets
for (j in 1:length(datasets)) {

  d <- datasets[j]
  print(paste0(d, ' : ', p))
  url <-URLencode(paste0("https://esoil.io/TERNLandscapes/SoilDataFederatoR/SoilDataAPI/SoilData?observedProperty=", p ,"&DataSet=", d, "&format=json&usr=", usr, "&key=", key))
  resp <- GET(url, timeout(300))
  response <- content(resp, "text", encoding = 'UTF-8')
  odf <- fromJSON(response)

  # If there is data returned slot it into the list
  if(is.data.frame(odf)){
    if(nrow(odf)>0){
      res[[j]] <- odf
    }
  }

}

# Merge the dataframes in the list into one dataframe
outDF = as.data.frame(data.table::rbindlist(res, fill=T))
# Write it to a csv
write.csv(outDF, paste0('c:/temp/SDF_', p, '.csv'))






